@@ -63,6 +63,16 @@ def test_equality():
     assert hash(face) != hash(face_alt)
 
 
+def test_face3d_no_holes():
+    """Test the initialization of Face3D without holes."""
+    pts = (Point3D(0, 0, 2), Point3D(0, 2, 2), Point3D(2, 2, 2), Point3D(2, 0, 2))
+    plane = Plane(Vector3D(0, 0, 1), Point3D(0, 0, 2))
+    face = Face3D(pts, plane, None)
+    assert not face.has_holes
+    face = Face3D(pts, plane, [])
+    assert not face.has_holes
+
+
 def test_face3d_to_from_dict():
     """Test the to/from dict of Face3D objects."""
     pts = (Point3D(0, 0, 2), Point3D(0, 2, 2), Point3D(2, 2, 2), Point3D(2, 0, 2))
@@ -572,6 +582,33 @@ def test_upper_left_counter_clockwise_vertices_triangle():
     assert up_cclock_1 == up_cclock_2
 
 
+def test_corner_vertices():
+    """Test the various properties for corner vertices."""
+    plane_1 = Plane(Vector3D(0, 1, 0))
+    plane_2 = Plane(Vector3D(0, -1, 0))
+    pts_1 = (Point3D(0, 0, 0), Point3D(2, 0, 0), Point3D(2, 0, 2), Point3D(0, 0, 2))
+    pts_2 = (Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 0, 0))
+    face_1 = Face3D(pts_1, plane_1)
+    face_2 = Face3D(pts_2, plane_1)
+    face_3 = Face3D(pts_1, plane_2)
+    face_4 = Face3D(pts_2, plane_2)
+
+    assert face_1.upper_left_corner.is_equivalent(Point3D(2, 0, 2), 1e-6)
+    assert face_1.lower_left_corner.is_equivalent(Point3D(2, 0, 0), 1e-6)
+    assert face_1.upper_right_corner.is_equivalent(Point3D(0, 0, 2), 1e-6)
+    assert face_1.lower_right_corner.is_equivalent(Point3D(0, 0, 0), 1e-6)
+
+    assert face_2.upper_left_corner.is_equivalent(Point3D(2, 0, 2), 1e-6)
+    assert face_2.lower_left_corner.is_equivalent(Point3D(2, 0, 0), 1e-6)
+    assert face_2.upper_right_corner.is_equivalent(Point3D(0, 0, 2), 1e-6)
+    assert face_2.lower_right_corner.is_equivalent(Point3D(0, 0, 0), 1e-6)
+
+    assert face_3.upper_left_corner.is_equivalent(Point3D(0, 0, 2), 1e-6)
+    assert face_3.lower_left_corner.is_equivalent(Point3D(0, 0, 0), 1e-6)
+    assert face_3.upper_right_corner.is_equivalent(Point3D(2, 0, 2), 1e-6)
+    assert face_3.lower_right_corner.is_equivalent(Point3D(2, 0, 0), 1e-6)
+
+
 def test_duplicate():
     """Test the duplicate method of Face3D."""
     pts = (Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 2, 2), Point3D(0, 2, 2))
@@ -1276,12 +1313,48 @@ def test_sub_faces_by_ratio_sub_rectangle():
     assert len(sub_faces) == 5
     areas = [srf.area for srf in sub_faces]
     assert sum(areas) == pytest.approx(face_1.area * 0.4, rel=1e-3)
+    for sf in sub_faces:
+        assert face_1.is_sub_face(sf, 0.01, 1)
 
     sub_faces_2 = face_1.sub_faces_by_ratio_sub_rectangle(
-        0.99, window_height, sill_height, horiz_separation, vert_separation, 0.0001)
-    assert len(sub_faces_2) == 2
+        0.95, window_height, sill_height, horiz_separation, vert_separation, 0.0001)
+    assert len(sub_faces_2) == 5
     areas = [srf.area for srf in sub_faces_2]
-    assert sum(areas) == pytest.approx(face_1.area * 0.99, rel=1e-3)
+    assert sum(areas) == pytest.approx(face_1.area * 0.95, rel=1e-3)
+    for sf in sub_faces_2:
+        assert face_1.is_sub_face(sf, 0.01, 1)
+
+
+def test_sub_faces_by_ratio_sub_rectangle_non_rect():
+    """Test the sub_faces_by_ratio_sub_rectangle method with a non-rectangular face."""
+    pts_1 = (Point3D(0, 0, 0), Point3D(5, 0, 0), Point3D(5, 0, 6), Point3D(0, 0, 5))
+    face_1 = Face3D(pts_1)
+
+    sub_faces = face_1.sub_faces_by_ratio_sub_rectangle(0.25, 2, 0.8, 3, 0, 0.01)
+    assert len(sub_faces) == 3
+    areas = [srf.area for srf in sub_faces]
+    assert sum(areas) == pytest.approx(face_1.area * 0.25, rel=1e-3)
+    for sf in sub_faces:
+        assert face_1.is_sub_face(sf, 0.01, 1)
+    
+    pts_2 = [
+        Point3D(34.068566268151841,  18.694085210192, 3.9999999999999987), 
+        Point3D(34.068566268151862, 18.694085210192011, 7.0), 
+        Point3D(34.442641391689776, 21.198643189820409, 7.0000000000000018), 
+        Point3D(34.442641391689783, 21.198643189820412, 7.4999999999999973), 
+        Point3D(33.710274088688173, 16.295199316291654, 7.5), 
+        Point3D(33.710274088688173, 16.295199316291651, 4.0)
+    ]
+    face_2 = Face3D(pts_2)
+
+    sub_faces = face_2.sub_faces_by_ratio_sub_rectangle(0.25, 2, 0.8, 3, 0, 0.01)
+    assert len(sub_faces) == 4
+    areas = [srf.area for srf in sub_faces]
+    assert all(a > 0.001 for a in areas)
+    assert sum(areas) == pytest.approx(face_2.area * 0.25, rel=1e-3)
+    for sf in sub_faces:
+        assert face_2.is_sub_face(sf, 0.01, 1)
+
 
 
 def test_sub_faces_by_ratio():
